@@ -33,6 +33,20 @@ _get_ves()
 				grep '^suspended ' | \
 				sed 's/^suspended *//' | $rm_empty
 			;;
+		convert)
+			# stopped CTs on simfs
+			(/usr/sbin/vzlist -HS -o layout,ctid;
+			 /usr/sbin/vzlist -HS -o layout,name) 2>/dev/null | \
+				grep '^simfs ' | \
+				sed 's/^simfs *//' | $rm_empty
+			;;
+		snapshot*)
+			# CTs on ploop
+			(/usr/sbin/vzlist -H -a -o layout,ctid;
+			 /usr/sbin/vzlist -H -a -o layout,name) 2>/dev/null | \
+				grep '^ploop ' | \
+				sed 's/^ploop *//' | $rm_empty
+			;;
 		*)
 			# All CTs
 			/usr/sbin/vzlist -H -a -octid 2>/dev/null
@@ -56,9 +70,11 @@ _vzctl()
 	local vzctl_common_opts="--quiet --verbose --help --version"
 	local vzctl_cmds="create destroy delete mount umount chkpnt restore \
 		set start stop restart status quotaon quotaoff quotainit \
-		enter exec exec2 runscript"
+		enter exec exec2 runscript convert console \
+		snapshot snapshot-switch snapshot-delete snapshot-list"
 	local vzctl_create_opts="--ostemplate --config --private --root \
-		--ipadd --hostname --name --description"
+		--layout --diskspace --ipadd --hostname --name --description"
+	local vzctl_convert_opts="--layout"
 	local vzctl_set_opts="--save --onboot --root --private \
 		--userpasswd --disabled --ipadd --ipdel \
 		--hostname --nameserver --searchdomain \
@@ -72,10 +88,13 @@ _vzctl()
 		--diskquota \
 		--diskspace --diskinodes --quotatime --quotaugidlimit \
 		--noatime --capability --devnodes --devices --applyconfig \
-		--name --ioprio --setmode --meminfo --bootorder --features"
+		--name --ioprio --setmode --meminfo --bootorder --features \
+		--reset_ub"
 	local vzctl_start_opts="--wait --force"
 	local vzctl_stop_opts="--fast"
 	local vzctl_restart_opts="$vzctl_start_opts $vzctl_stop_opts"
+	local vzctl_snapshot_create_opts="--id --name --description \
+		--skip-suspend"
 
 	local iptables_names="iptable_filter iptable_mangle ipt_limit
 		ipt_multiport ipt_tos ipt_TOS ipt_REJECT ipt_TCPMSS
@@ -126,8 +145,8 @@ _vzctl()
 			--ostemplate)
 				# get the template path from the vz.conf
 				local vztmpl=`grep ^TEMPLATE /etc/vz/vz.conf | cut -d "=" -f 2`
-				COMPREPLY=( $( compgen -W "$(command ls -1 $vztmpl/cache/*.tar.gz 2>/dev/null |
-						    sed -e "s#^$vztmpl/cache/##" -e 's#.tar.gz$##')" -- $cur ) )
+				COMPREPLY=( $( compgen -W "$(command ls $vztmpl/cache/*.tar{,.gz,.xz,.bz2} 2>/dev/null |
+						    sed -e "s#^$vztmpl/cache/##" -e 's#\.tar\.*[gbxz2]*$##')" -- $cur ) )
 
 				;;
 			--onboot|--disabled|--noatime|--diskquota)
@@ -142,6 +161,9 @@ _vzctl()
 
 				COMPREPLY=( $( compgen -W "$configs" -- $cur ) )
 				;;
+			--layout)
+				COMPREPLY=( $( compgen -W "ploop ploop:expanded ploop:plain ploop:raw simfs" -- $cur ) )
+				;;
 			--iptables)
 				COMPREPLY=( $( compgen -W "$iptables_names" -- $cur ) )
 				;;
@@ -246,6 +268,9 @@ _vzctl()
 					set)
 						COMPREPLY=( $( compgen -W "$vzctl_set_opts" -- $cur ) )
 						;;
+					convert)
+						COMPREPLY=( $( compgen -W "$vzctl_convert_opts" -- $cur ) )
+						;;
 					chkpnt|restore)
 						COMPREPLY=( $( compgen -W "--dumpfile" -- $cur ) )
 						;;
@@ -258,6 +283,12 @@ _vzctl()
 					restart)
 						COMPREPLY=( $( compgen -W "$vzctl_restart_opts" -- $cur ) )
 						;;
+					snapshot)
+						COMPREPLY=( $( compgen -W "$vzctl_snapshot_create_opts" -- $cur ) )
+						;;
+					snapshot-switch|snapshot-delete)
+						COMPREPLY=( $( compgen -W "--id" -- $cur ) )
+						;;
 					*)
 						;;
 					esac
