@@ -1,5 +1,5 @@
-#!/bin/bash
-#  Copyright (C) 2000-2011, Parallels, Inc. All rights reserved.
+#!/bin/sh
+#  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,21 +16,32 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 #
-# runs on the host system and performs these postinstall tasks:
-# 1. Randomizes crontab for given container so all crontab tasks
+# Runs on the host system and performs these postinstall tasks
+# for a particular container (specified by $VE_ROOT environment):
+#
+# 1. Randomizes /etc/crontab and /etc/cron.d/* files so all crontab tasks
 #    of all containers will not start at the same time.
+#    NOTE: if you don't want a particular file to be randomized,
+#    make sure it contains the word VZCTL_POSTCREATE_SKIP somewhere.
+#
 # 2. Disables root password if it is empty.
+#
 # 3. Seeds /etc/hosts with localhost entries for IPv4 and IPv6
+#
 # 4. Creates empty /etc/resolv.conf
+#
+# 5. Sets file caps for recent Fedora releases, as they can
+#    not be saved in a template tarball.
 
-function randcrontab()
+randcrontab()
 {
-	local file=${VE_ROOT}"/etc/crontab"
+	local file
+	for file in ${VE_ROOT}/etc/crontab ${VE_ROOT}/etc/cron.d/*; do
+		[ -f "${file}" ] || continue
+		grep -wq VZCTL_POSTCREATE_SKIP "${file}" && continue
 
-	[ -f "${file}" ] || return 0
-
-	/bin/cp -fp ${file} ${file}.$$
-	cat ${file} | awk '
+		/bin/cp -fp ${file} ${file}.$$
+		cat ${file} | awk '
 BEGIN { srand(); }
 {
 	if ($0 ~ /^[ \t]*#/ || $0 ~ /^[ \t]+*$/) {
@@ -61,12 +72,12 @@ BEGIN { srand(); }
 		line = line " "  ar[i];
 	}
 	print line;
-}
-' > ${file}.$$ && /bin/mv -f ${file}.$$ ${file}
-	/bin/rm -f ${file}.$$ 2>/dev/null
+}' >		${file}.$$ && /bin/mv -f ${file}.$$ ${file}
+		/bin/rm -f ${file}.$$ 2>/dev/null
+	done
 }
 
-function disableroot()
+disableroot()
 {
 	local file=${VE_ROOT}"/etc/passwd"
 
@@ -79,7 +90,7 @@ function disableroot()
 	fi
 }
 
-function set_network()
+set_network()
 {
 	local file=${VE_ROOT}"/etc/hosts"
 	if ! grep -qw '127.0.0.1' ${file} 2>/dev/null; then
@@ -97,9 +108,30 @@ function set_network()
 	fi
 }
 
+_sc()
+{
+	local val=$1
+	shift
+	local file
+	for file in $*; do
+		setfattr -n security.capability -v $val ${VE_ROOT}${file}
+	done
+}
+
+set_file_caps()
+{
+	# Perform this only for Fedora 15 to 19
+	grep -qEw '1[5-9]' ${VE_ROOT}/etc/fedora-release 2>/dev/null || return
+
+	_sc 0sAQAAAgkAAAAAAAAAAAAAAAAAAAA= /usr/libexec/pt_chown
+	_sc 0sAQAAAsIAAAAAAAAAAAAAAAAAAAA= /usr/sbin/suexec
+	_sc 0sAQAAAgAgAAAAAAAAAAAAAAAAAAA= /bin/ping /bin/ping6
+}
+
 [ -z "${VE_ROOT}" ] && exit 1
 randcrontab
 disableroot
 set_network
+set_file_caps
 
 exit 0
